//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  Cloak.cpp
//
//  The actually cloak going over UOX, handles thread controll, piping etc.
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#include "stdafx.h"
#include <string>
#include "Cloak.h"
#include "ConsoleView.h"
#include "winsock2.h"
#include "MainFrm.h"
#include "default.h"
#include "commands.h"

bool          Cloak::scoop_running   = false;
bool          Cloak::scoop_done      = false;
bool          Cloak::gurdian_running = false;
unsigned long Cloak::scoop_id        = 0;
unsigned long Cloak::gurdian_id      = 0;

HANDLE        Cloak::hpipe_out_read  = 0;
HANDLE        Cloak::hpipe_out_write = 0;
HANDLE        Cloak::hpipe_in_read   = 0;
HANDLE        Cloak::hpipe_in_write  = 0;
HANDLE        Cloak::huox            = 0;

bool          Cloak::shutdown_request = false;

bool          Cloak::cloak_on   = false;
bool          Cloak::cloak_done = false;
time_t        Cloak::shutdown_start = NULL;
time_t        Cloak::watchdog_start = NULL;


double        max_shutdown_time = 60;      // server has 1 minute to shutdown
/*
double        watchdog_time     = 60 * 10; // watchdog checks server all 10 minutes
unsigned int  watchdog_space    = 60;      // 60 seconds between the watchdog retires
*/

MainFrame   * Cloak::mainFrame   = NULL;
ConsoleView * Cloak::consoleView = NULL;
bool          Cloak::first_active;


int  Cloak::check_delay   = DEFAULT_CHECK_DELAY;   // minutes between checks
int  Cloak::retries       = DEFAULT_RETRIES;       // retry count
int  Cloak::first_check   = DEFAULT_FIRST_CHECK;   // additional time until first check
int  Cloak::response_time = DEFAULT_RESPONSE_TIME; // server respond time
int  Cloak::retry_delay   = DEFAULT_RETRY_DELAY;   // server respond time
bool Cloak::watchdog_enable = true;				   // is watchdog enabled?
bool Cloak::logtofile_enabled = true;			   // do we want to log all console messages?
CString Cloak::logfile_directory = "";			   // where to log to
CString Cloak::uox_directory = "";				   // where is uox at
bool Cloak::autostart_server = false;			   // do we want uox to start automatically
bool Cloak::server_is_running = false;			   // is uox running already?
bool Cloak::secure_mode = true;					   // is uox server in secure mode?

void Cloak::initalize(ConsoleView *s_consoleView, MainFrame *s_mainFrame)
{
	consoleView = s_consoleView;
    mainFrame   = s_mainFrame;

}

void Cloak::finalize()
{
  cloak_on = false;
}

void Cloak::execute()
{ 
	if(autostart_server){
		if(!logtofile_enabled){
			consoleView->string_in("==Console Logging Off==\n\n", BLUE);
		} else {
			consoleView->string_in("==Console Logging On==\n\n", BLUE);
		}
		consoleView->string_in("--- STARTING SERVER ---\n", BLUE);
		if (!probe_uox_server_port()) {
			return;
		}
  
		SECURITY_ATTRIBUTES sa = {0, };
		STARTUPINFO         si = {0, };
		PROCESS_INFORMATION pi = {0, };
      
		sa.nLength = sizeof(sa);
		sa.bInheritHandle = true;
		sa.lpSecurityDescriptor = NULL;

		// create the pipes
		CreatePipe(&hpipe_out_read, &hpipe_out_write, &sa, 0);
		CreatePipe(&hpipe_in_read,  &hpipe_in_write,  &sa, 0);
  
		si.cb = sizeof(si);
		si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
		si.wShowWindow = SW_HIDE;
		si.hStdInput   = hpipe_in_read;
		si.hStdOutput  = hpipe_out_write;
		si.hStdError   = hpipe_out_write;
		si.dwXCountChars = 0x47252918;
		si.dwYCountChars = 0x88531920;
  
		// create to uox process
		CString parms = uox_directory;
		parms += "uox3.exe -cluox100 ";
		CString workingdir = uox_directory;
		workingdir += " ";
		CString uoxdirerror = "Current UOX3 Directory: ";
		uoxdirerror += parms;
		char itoabuf[255];
		itoa((unsigned long) hpipe_in_write, itoabuf, 16);
		parms += itoabuf;
		parms.LockBuffer();
		unsigned int asw = CreateProcess(NULL, parms.GetBuffer(0), NULL, NULL, TRUE, 0, NULL, workingdir.GetBuffer(0), &si, &pi);
		huox = pi.hProcess;
		parms.UnlockBuffer();

		if (asw == 0) {
			consoleView->error("Cannot create UOX sub-process.", uoxdirerror, NULL);
			return;
		}
		// set the watchdog
		time(&watchdog_start); 

		// now create a thread that scoops uox's output stream in to the consoleview.
		HANDLE scoop_thread; 
		scoop_thread = CreateThread(&sa, 0, scoop_start, 0, 0, &scoop_id); 

		// create the gurdian thread that watches over uox.
		HANDLE gurdian_thread; 
		gurdian_thread = CreateThread(&sa, 0, gurdian_start, 0, 0, &gurdian_id); 
		first_active = true;
		cloak_on = true;
		server_is_running = true;
	}
};

void Cloak::restart()
{
	if(autostart_server || server_is_running){
		if (!probe_uox_server_port()) {
			return;
		}
		consoleView->string_in("--- RESTARTING SERVER ---\n", BLUE);

		// restart the uox process using exactly the same pipes
		SECURITY_ATTRIBUTES sa = {0, };
		STARTUPINFO         si = {0, };
		PROCESS_INFORMATION pi = {0, };
      
		sa.nLength = sizeof(sa);
		sa.bInheritHandle = true;
		sa.lpSecurityDescriptor = NULL;
 
		si.cb = sizeof(si);
		si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
		si.wShowWindow = SW_HIDE;
		si.hStdInput   = hpipe_in_read;
		si.hStdOutput  = hpipe_out_write;
		si.hStdError   = hpipe_out_write;

		CString parms = uox_directory;
		parms += "uox3.exe -cluox100 ";
		CString workingdir = uox_directory;
		workingdir += " ";
		CString uoxdirerror = "Current UOX3 Directory: ";
		uoxdirerror += parms;
		char itoabuf[255];
		itoa((unsigned long) hpipe_in_write, itoabuf, 16);
		parms += itoabuf;
		parms.LockBuffer();
		unsigned int asw = CreateProcess(NULL, parms.GetBuffer(0), NULL, NULL, TRUE, 0, NULL, workingdir.GetBuffer(0), &si, &pi);
		huox = pi.hProcess;
		parms.UnlockBuffer();

		if (asw == 0) {
			consoleView->error("Cannot create UOX sub-process.", uoxdirerror, NULL);
			return;
		}  
		// reset the watchdog
		first_active = true;
		time(&watchdog_start);
	}
}
 
unsigned long Cloak::scoop_start(void *parm)
{
  scoop_running = true;
  scoop_done    = false;

  char buf[256];
  unsigned int bufp = 0; 
  char ch;
  int asw;
  unsigned long bytes_read;
  while (scoop_running) {
    // do something
    asw = ReadFile(hpipe_out_read, &ch, 1, &bytes_read, NULL);
    buf[bufp] = ch;
    bufp++;
    if ((bufp >= 255) || (ch == 13) || (ch == 10)) {
      consoleView->string_in(buf, WHITE, bufp);
      bufp = 0;
    }
  }
  scoop_done = true;
  return 0;
}

unsigned long Cloak::gurdian_start(void *parm)
{
  gurdian_running = true;
  while (gurdian_running) {
    Sleep(100);
    time_t now;
    time(&now);
    unsigned long exitcode;
    GetExitCodeProcess(huox, &exitcode);
    if (exitcode != STILL_ACTIVE) {
      // uox has terminated!
      huox = 0;
      if (shutdown_request) {
        // this okay, say the mainform it is finished now.
        terminate_cloak();
        mainFrame->SendMessage(WM_CLOSE, 0, 0);
        return 0;
      }
      restart(); // the basard :>
    }
    if (shutdown_request) {
      double elapsed_time = difftime(now, shutdown_start);
      if (elapsed_time > max_shutdown_time) {
        int asw = MessageBox(mainFrame->m_hWnd, "Shutdown timeout expired...\nShall I kill the UOX Process?", "cluox", MB_YESNO);
        if (asw == IDYES) {
          terminate_cloak();
          mainFrame->SendMessage(WM_CLOSE, 0, 0);
        } else {
          // restart timeout
          time(&shutdown_start);
        }
      }
    }
    // do watch-dog-ing
    if (watchdog_enable) {
      double elapsed_time = difftime(now, watchdog_start);
      double watchdog_time = check_delay * 60;
      if (first_active) {
        watchdog_time += first_check * 60;
      }
      if (elapsed_time > watchdog_time) {
        first_active = false;
        check_watchdog();
      }
    }
  }
  return 0;
}

void Cloak::feed_char(unsigned char ch)
{
  if (!cloak_on) {
    return;
  }
  unsigned long bytes_written;
  WriteFile(hpipe_in_write, &ch, 1, &bytes_written, NULL);
}

void Cloak::shutdown()
{
  if (shutdown_request) {
    return;
  }
  consoleView->string_in("\nSHUTDOWN\n", BLUE);
  feed_char(250);
  Sleep(100);
  feed_char('Q');
  time(&shutdown_start);
  shutdown_request = true;
}

void Cloak::check_watchdog() 
{
  if (shutdown_request) {
    return;
  }
  consoleView->string_in("\nWatchdog: probing UOX client port ...\n", BLUE);
  bool watchdog = probe_uox_client_port();
  if (watchdog) {
    // uox okay
    //Sleep(3000);
    consoleView->string_in("Watchdog: OKAY\n", BLUE);
    // restart timeout
    time(&watchdog_start);
    return;
  } 
  // uox hanged
  for(unsigned int i = 0; i < retries; i++) {
    CString msg;
    msg.Format("Watchdog: FAILURE %d of %d", i + 1, retries);
    msg.LockBuffer();
    consoleView->string_in(msg.GetBuffer(0), BLUE);
    msg.UnlockBuffer();
    if (i + 1 == retries) {
      break;
    }
    Sleep(retry_delay * 1000); 
    if (shutdown_request) {
      break;
    }
    consoleView->string_in(" RETRYING ...\n", BLUE);
    watchdog = probe_uox_client_port();
    if (watchdog) {
      break;
    }
  }
  if (!watchdog) {
    // still hangen after retries -> termiante.
    consoleView->string_in("\nWatchdog: FAILURE, TERMINATING SERVER!\n", RED);
    terminate_uox();
    Sleep(5000); // give him 5 seconds to really terminate.
    if (shutdown_request) {
      return;
    }
    restart();
  } else {
    if (!shutdown_request) {
      Sleep(3000);
      consoleView->string_in("\nWatchdog: OKAY\n", BLUE);
      // restart timeout
      time(&watchdog_start);
    }
  }
}

void Cloak::terminate_cloak()
{
  if (!cloak_on) {
    return;
  }
  cloak_on = false;
  gurdian_running = false;
  if (scoop_running) {
    scoop_running = false;
    // terminate the current scooper thread.
    // give the thread one char to relief it's blocking
    unsigned long bytes_written;
    unsigned char ch = 0;
    WriteFile(hpipe_out_write, &ch, 1, &bytes_written, NULL);  
    // don't know, why it isn't neccessary to wait, it seems the scoop thread terminates
    // itself... maybe something throws an exception? 
    ASSERT(bytes_written == 1);
    while (!scoop_done) {
      Sleep(100);
    }
  }

  if (huox != 0) {
    // terminate UOX if it's running
    TerminateProcess(huox, -1);
    // don't care if this succedes, just try once. 
    huox = 0;
  }

  // close the pipes
  CloseHandle(hpipe_out_read);
  CloseHandle(hpipe_out_write);
  CloseHandle(hpipe_in_read);
  CloseHandle(hpipe_in_write);
  
  cloak_done = true;
}


// opens a test server socket on 2593 to check if uox is capable to run
bool Cloak::probe_uox_server_port()
{
  SOCKET s;
  WSADATA wsaData;
  unsigned int version_request = MAKEWORD(2, 0);
  if (WSAStartup(version_request, &wsaData)) {
    consoleView->error("Cannot find WINSOCK2, get an update at microsoft", "(free of charge)", NULL);
  }
  s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

  if (s >= 0) {
    struct sockaddr_in connection = {0, };
    connection.sin_family=AF_INET;
    connection.sin_addr.s_addr=htonl(INADDR_ANY);
    connection.sin_port=htons(2593);

    int asw = bind(s, (struct sockaddr *)&connection, sizeof (struct sockaddr_in));
    if (asw >= 0) {
      closesocket(s);
      return true;
    }
  }
  // error occured
  terminate_cloak();
  consoleView->error("There is already an UO-Server running, UOX not started", 
                     "(Probing of port 2593 failed)", 
                     "(maybe also check in your Task-Manager if an invisilbe UOX instance is left running)"
                    );

  return false;
}

// opens a client socket on 2593 to check if uox is running right in the moment
bool Cloak::probe_uox_client_port()
{
  SOCKET s;
  unsigned int version_request = MAKEWORD(2, 0);
  s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

  if (s < 0) {
    return false;
  }
  struct sockaddr_in connection = {0, };
  connection.sin_family=AF_INET;
  //connection.sin_addr.s_addr=htonl(INADDR_ANY);
  connection.sin_addr.s_addr = INADDR_ANY;
  //connection.sin_port=htons(0);
  connection.sin_port=htons(0);

  int asw = bind(s, (struct sockaddr *)&connection, sizeof (struct sockaddr_in));
  if (asw < 0) {
    return false;
  }

  struct sockaddr_in remote = {0, };
  remote.sin_family=AF_INET;
  remote.sin_addr.s_addr=htonl(0x7f000001); // 127.0.0.1
  remote.sin_port=htons(2593);

  fd_set set = {0, };
  FD_ZERO(&set);
  FD_SET(s, &set);
  struct timeval timeout;
  timeout.tv_sec  = response_time;
  timeout.tv_usec = 0;

  asw = connect(s, (struct sockaddr *) &remote, sizeof (struct sockaddr));
  if (asw != 0) {
    return false;
  }
  asw = select(0, NULL, &set, NULL, &timeout);
  if (asw <= 0) {
    closesocket(s);
    return false;
  }

  const unsigned char loginmsg[4 + 62] = 
      {0, 100, 1, 0,
       0x80, 
       'W','A','T','C','H','D','O','G',0,0,
       0,0,0,0,0,0,0,0,0,0,
       0,0,0,0,0,0,0,0,0,0,
       'W','A','T','C','H','D','O','G',0,0,
       0,0,0,0,0,0,0,0,0,0,
       0,0,0,0,0,0,0,0,0,0,
       0xff,
      };
  asw = send(s, (const char *) loginmsg, 4 + 62, 0);
  if (asw != 4 + 62) {
    closesocket(s);
    return false;
  } 
  asw = select(0, &set, NULL, NULL, &timeout);
  if (asw <= 0) {
    closesocket(s);
    return false;
  }
  closesocket(s);
  return true;
}

void Cloak::terminate_uox() 
{
  TerminateProcess(huox, -1);
  // don't care if this succedes, just try once. 
  huox = 0;
}

void Cloak::startserver()
{
	if(server_is_running){ return; }
	if(!logtofile_enabled){
		consoleView->string_in("==Console Logging Off==\n\n", BLUE);
	} else {
		consoleView->string_in("==Console Logging On==\n\n", BLUE);
	}
	consoleView->string_in("--- STARTING SERVER ---\n", BLUE);
	if (!probe_uox_server_port()) {
		return;
	}
  
	SECURITY_ATTRIBUTES sa = {0, };
	STARTUPINFO         si = {0, };
	PROCESS_INFORMATION pi = {0, };
      
	sa.nLength = sizeof(sa);
	sa.bInheritHandle = true;
	sa.lpSecurityDescriptor = NULL;

	// create the pipes
	CreatePipe(&hpipe_out_read, &hpipe_out_write, &sa, 0);
	CreatePipe(&hpipe_in_read,  &hpipe_in_write,  &sa, 0);
  
	si.cb = sizeof(si);
	si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
	si.wShowWindow = SW_HIDE;
	si.hStdInput   = hpipe_in_read;
	si.hStdOutput  = hpipe_out_write;
	si.hStdError   = hpipe_out_write;
	si.dwXCountChars = 0x47252918;
	si.dwYCountChars = 0x88531920;
  
	// create to uox process
	CString parms = uox_directory;
	parms += "uox3.exe -cluox100 ";
	CString workingdir = uox_directory;
	workingdir += " ";
	CString uoxdirerror = "Current UOX3 Directory: ";
	uoxdirerror += parms;
	char itoabuf[255];
	itoa((unsigned long) hpipe_in_write, itoabuf, 16);
	parms += itoabuf;
	parms.LockBuffer();
	unsigned int asw = CreateProcess(NULL, parms.GetBuffer(0), NULL, NULL, TRUE, 0, NULL, workingdir.GetBuffer(0), &si, &pi);
	huox = pi.hProcess;
	parms.UnlockBuffer();

	if (asw == 0) {
		consoleView->error("Cannot create UOX sub-process.", uoxdirerror, NULL);
		return;
	}
	// set the watchdog
	time(&watchdog_start); 

	// now create a thread that scoops uox's output stream in to the consoleview.
	HANDLE scoop_thread; 
	scoop_thread = CreateThread(&sa, 0, scoop_start, 0, 0, &scoop_id); 

	// create the gurdian thread that watches over uox.
	HANDLE gurdian_thread; 
	gurdian_thread = CreateThread(&sa, 0, gurdian_start, 0, 0, &gurdian_id); 
	first_active = true;
	cloak_on = true;
	server_is_running = true;
}

void Cloak::reloadaccounts()
{
	consoleView->string_in("\nRELOADING ACCOUNTS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadAccounts);
}

void Cloak::reloaddfnfiles()
{
	consoleView->string_in("\nRELOADING DFN FILES\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadDFNFiles);
}
void Cloak::reloadini()
{
	consoleView->string_in("\nRELOADING INI FILE\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadINI);
}
void Cloak::reloadjsfiles()
{
	consoleView->string_in("\nRELOADING JS FILES\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadJSFiles);
}
void Cloak::reloadregions()
{
	consoleView->string_in("\nRELOADING REGIONS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadRegions);
}
void Cloak::reloadspawnregions()
{
	consoleView->string_in("\nRELOADING SPAWN REGIONS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadSpawnRegions);
}
void Cloak::reloadspells()
{
	consoleView->string_in("\nRELOADING SPELLS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ReloadSpells);
}

void Cloak::secureconsole()
{
	if(!secure_mode){
		consoleView->string_in("\nSECURING CONSOLE\n", BLUE);
		feed_char(SecureUOX);
		secure_mode = true;
	}
}

void Cloak::commandlist()
{
	consoleView->string_in("\nSHOWING COMMAND LIST\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(CommandList);
}

void Cloak::configuration()
{
	consoleView->string_in("\nSHOWING CONFIGURATION\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(Configuration);
}

void Cloak::consolebroadcast()
{
	consoleView->string_in("\nCONSOLE BROADCAST\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(ConsoleBroadcast);
}

void Cloak::save_accounts()
{
	consoleView->string_in("\nSAVING ACCOUNTS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(SaveAccounts);
}

void Cloak::save_world()
{
	consoleView->string_in("\nSAVING WORLD\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(SaveWorld);
}

void Cloak::maintain_performance()
{
	consoleView->string_in("\nSTARTING PERFORMANCE DUMP\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainPerformance);
}

void Cloak::maintain_characters()
{
	consoleView->string_in("\nDUMPING USERS ONLINE\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainCharacters);
}

void Cloak::maintain_memory()
{
	consoleView->string_in("\nDUMPING MEMORY USAGE\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainMemory);
}

void Cloak::maintain_shutdown()
{
	consoleView->string_in("\nTEN MINUTE SHUTDOWN STARTED\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainShutdown);
}

void Cloak::maintain_lookups()
{
	consoleView->string_in("\nDUMPING LOOKUPS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainDumpLookups);
}

void Cloak::maintain_maps()
{
	consoleView->string_in("\nDUMPING PRIORITY MAPS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(MainDisplayMaps);
}

void Cloak::network_disconzero()
{
	consoleView->string_in("\nDISCONNECTING ACCOUNT ZERO\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(DisconnectAcctZero);
}

void Cloak::network_disconall()
{
	consoleView->string_in("\nDISCONNECTING ALL ACCOUNTS\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(DisconnectAll);
}

void Cloak::network_socketlogging()
{
	consoleView->string_in("\nLOGGING SOCKET\n", BLUE);
	if(secure_mode){
		feed_char(SecureUOX);
		secure_mode = false;
		Sleep(100);
	}
	feed_char(SocketLogging);
}

