//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  ConBuffer.cpp
//
//  The sub-view of the black console, handles the cursor position amd also
//  keeps the screen buffer.
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#include "stdafx.h"
#include "ConBuffer.h"
#include "default.h"

ConBuffer::ConBuffer()
{
  for(unsigned int y = 0; y < MAX_CONSOLE_LINES; y++) {
    for(unsigned int x = 0; x <= 80; x++) {
      screen[y][x] = ' ';
      color[y][x] = 0;
    }
  }
  min = 0;
  max = 1;
}

const char *ConBuffer::getline(unsigned int y)
{
  //ASSERT(y <= max - min);
  y %= MAX_CONSOLE_LINES;
  return screen[y];
}

const unsigned int *ConBuffer::getcolor(unsigned int y)
{
  //ASSERT(y <= max - min);
  y %= MAX_CONSOLE_LINES;
  return color[y];
}

void ConBuffer::setchar(unsigned int x, unsigned int y, char ch, unsigned int color)
{
  //ASSERT(y <= max - min);
  y %= MAX_CONSOLE_LINES;
  screen[y][x] = ch;
  this->color[y][x] = color;
}

void ConBuffer::newline()
{
  max++;
  if (max > MAX_CONSOLE_LINES) {
    min++;
  }
  int y = max % MAX_CONSOLE_LINES;
  // erase new line
  for (int x = 0; x <= 80; x++) {
    screen[y][x] = ' ';
    color[y][x] = WHITE;
  }
}

unsigned int ConBuffer::size()
{
  return max - min;
}
