//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  ConsoleView.cpp
//
//  The sub-view of the black console, handles the cursor position 
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#include "stdafx.h"
#include <string>
#include <time.h>
#include "cluox.h"
#include "Cloak.h"
#include "ConsoleView.h"
#include "MainFrm.h"

ConsoleView::ConsoleView()
{
  if(Cloak::logtofile_enabled){
	resized = false;
	cursx = 0;
	cursy = 0;
	viewy = 0;
	hotview = true;

	_error = false;
	lastlf = false;
  
	CString logfile = Cloak::logfile_directory;
	logfile += "cluox_log.log";
	log = fopen(logfile, "a");
	if (log == NULL) {
	  _error = true;
	  err1 = "Cannot open \"cluox_log.log\" for logging.";
	  err2 = "";
	  err3 = "";  
	}
	fputc('\n', log);
	do_log('>', "******************************************************************************");
	std::string msg = " cluox session started at ";
	msg += _strdate(times);
	msg += " ";
	msg += _strtime(times);
	do_log('>', msg.c_str());
	do_log('>', "******************************************************************************");
	do_log(' ', "");
  }
}

ConsoleView::~ConsoleView()
{  
  Cloak::finalize();
  if (log != NULL) {
    fclose(log);
  }
}


BEGIN_MESSAGE_MAP(ConsoleView,CWnd )
	//{{AFX_MSG_MAP(ConsoleView)
	ON_WM_PAINT()
	ON_WM_VSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL ConsoleView::PreCreateWindow(CREATESTRUCT& cs) 
{
  if (!CWnd::PreCreateWindow(cs)) {
    return FALSE;
  }

  cs.dwExStyle |= WS_EX_CLIENTEDGE;
  cs.style &= ~WS_BORDER;
  cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
                 ::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);

  return TRUE;
}

void ConsoleView::OnPaint() 
{
  if (conbuf.getmax() <= 24) {
    scrollInfo.nPage = 2000; // gives enough illusion of a full scrollbar
    scrollInfo.nMin  = 0;
    scrollInfo.nMax  = 2000; 
    scrollInfo.fMask = SIF_ALL;
    SetScrollInfo(SB_VERT, &scrollInfo, false);
    SetScrollPos(SB_VERT, viewy, true);
  } else {
    scrollInfo.nPage = 24;
    scrollInfo.nMin  = conbuf.getmin();
    scrollInfo.nMax  = conbuf.getmax() - 1;
    scrollInfo.fMask = SIF_ALL;
    SetScrollInfo(SB_VERT, &scrollInfo, false);
    SetScrollPos(SB_VERT, viewy, true);
  }
  if (!resized) {
    resized = true;
    cloak->execute();
  }
  CRect rect;
  GetClientRect(rect);
  CPaintDC dc(this); // device context for painting
  
  if (_error) {
    dc.SetBkMode(TRANSPARENT);
    dc.TextOut(40,  40, "ERROR:");
    dc.TextOut(70,  60, err1);
    dc.TextOut(70,  80, err2);
    dc.TextOut(70, 100, err3);
    return;
  }

  //dc.SetTextColor(0xffffff);
  dc.SetBkColor(0);
  dc.SelectObject(console_font);
  char buf[2] = {0,};
  
  for(unsigned int y = 0; y < 24; y++) {
    int drawy = y + viewy;
    if (drawy <= conbuf.getmax()) {
      const unsigned int *colors = conbuf.getcolor(drawy);
      const char *        chars  = conbuf.getline(drawy);
      for (unsigned int x = 0; x < 80; x++) {
        dc.SetTextColor(colors[x]);
        buf[0] = chars[x];
        dc.TextOut(x * x_spacing, y * line_spacing, buf);
      }
    } else {
      for (unsigned int x = 0; x < 80; x++) {
        dc.SetTextColor(0);
        buf[0] = ' ';
        dc.TextOut(x * x_spacing, y * line_spacing, buf);
      }
    }
  }
}

void ConsoleView::resize() 
{

  CPaintDC dc(this);
  dc.SelectObject(console_font);
  TEXTMETRIC metrics;
  dc.GetTextMetrics(&metrics);

  CRect wrect;
  CRect crect;

  GetParent()->GetWindowRect(&wrect);
  GetClientRect(&crect);

  int borderw = wrect.Width()  - crect.Width();
  int borderh = wrect.Height() - crect.Height();
  x_spacing = metrics.tmAveCharWidth;
  line_spacing = metrics.tmHeight;
  GetParent()->MoveWindow(wrect.left, wrect.top, (int) (metrics.tmAveCharWidth * 80 + borderw), (int) (metrics.tmHeight * 24 + borderh));
}

void ConsoleView::char_in(char ch)
{
  mutex_char_in.Lock();
  do_in(ch, WHITE);
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
  mutex_char_in.Unlock();
}

void ConsoleView::char_in(char ch, unsigned int color)
{
  mutex_char_in.Lock();
  do_in(ch, color);
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
  mutex_char_in.Unlock();
}

void ConsoleView::string_in(const char *str, unsigned int color, unsigned int len)
{
  mutex_char_in.Lock();
  for (unsigned i = 0; i < len; i++) {
    do_in(str[i], color);
  }
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
  mutex_char_in.Unlock();
}

void ConsoleView::string_in(const char *str, unsigned int color)
{
  mutex_char_in.Lock();
  for (unsigned i = 0; str[i] != 0; i++) {
    do_in(str[i], color);
  }
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
  mutex_char_in.Unlock();
}

void ConsoleView::string_in(const char *str)
{
  mutex_char_in.Lock();
  for (unsigned i = 0; str[i] != 0; i++) {
    do_in(str[i], WHITE);
  }
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
  mutex_char_in.Unlock();
}

void ConsoleView::do_in(char ch, unsigned int c_color)
{
  if (lastlf && (ch == 10)) {
    lastlf = false;
    return; // do nothing for lf on cr/lf
  } 
  if (ch == 13) {
    lastlf = true;  
  } else {
    lastlf = false;
  }
  if (ch == 9) {
    // tab transorm into 8 spaces
    for(unsigned int i = 0; i < 8; i++) {
      do_in(' ', c_color);
    }    
    return;
  }
  int post_ch = -1;
  if ((ch == 13) || (ch == 10)) {
    // carride return or line feed
    cursx = 0;
    newline();
  } else if (ch == 8) {
    // backspace
    if (cursx > 0) {
     conbuf.setchar(--cursx, cursy, ' ', c_color);
    }    
  } else {
    if (cursx >= 80) {      
      cursx = 0;
      newline();
      if (cursy < MAX_CONSOLE_LINES) {
        conbuf.setchar(cursx++, cursy, ch, c_color);
      } else {
        post_ch = ch;
        cursx++;
      }
    } else {
      conbuf.setchar(cursx++, cursy, ch, c_color);
    }
  }
  /*
  if (cursy >= MAX_CONSOLE_LINES) {
    unsigned int x;
    unsigned int y;
    for (y = 1; y < MAX_CONSOLE_LINES; y++) {
      for (x = 0; x < 80; x++) {
        screen[y - 1][x] = screen[y][x];
        color[y - 1][x] = color[y][x];
      }
    }
    for (x  = 0; x < 80; x++) {
      screen[MAX_CONSOLE_LINES - 1][x] = ' ';
      color[MAX_CONSOLE_LINES - 1][x] = c_color;
    }
    cursy--;
    if (post_ch >= 0) {
      screen[MAX_CONSOLE_LINES - 1][0] = post_ch;
      color[MAX_CONSOLE_LINES - 1][0] = c_color;
    }
  }
  */
}

void ConsoleView::newline()
{
  char kind = ' ';
  switch (conbuf.getcolor(cursy)[0]) {
    case WHITE : kind = '|'; break;
    case BLUE  : kind = '+'; break;
    case RED   : kind = '>'; break;
  }
  do_log(kind, conbuf.getline(cursy));
  cursy++;
  if (hotview && (conbuf.getmax() >= 24)) {
    viewy++;
  }
  conbuf.newline();
  if (viewy < conbuf.getmin()) {
    viewy = conbuf.getmin();
  }
}


void ConsoleView::error(const char *text1, const char *text2, const char *text3) 
{
  err1 = text1;
  err2 = text2;
  err3 = text3;  
  _error = true;
  RedrawWindow();
}

void ConsoleView::do_log(char kind, const char *text)
{
	if(Cloak::logtofile_enabled){
		int len = strlen(text);
		while (len > 0) {
			if (text[len - 1] != ' ') {
				break;
			}
			len--;
		}
		fputs(_strtime(times), log);
		fputc(' ',  log);
		fputc(kind, log);
		fputc(' ',  log);
		for(unsigned int i = 0; i < len; i++) {
			const char ch = text[i];
			if ((ch != '\n') && (ch != '\r')) {
				fputc(ch, log);
			}
		}
		fputc('\n',  log);
		fflush(log);
	}
}


void ConsoleView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
  CWnd::OnVScroll(nSBCode, nPos, pScrollBar);
  if (conbuf.getmax() <= 24) {
    return; // disabled on startscreen
  }
  hotview = false; // do not autoscroll
  switch (nSBCode) {
    case SB_BOTTOM :    // Scrolls to the lower right. 
        viewy = cursy - 24;
        break;
    case SB_ENDSCROLL : // Ends scroll. 
        break;
    case SB_LINEDOWN :  // Scrolls one line down. 
        if (viewy < conbuf.getmax()) {
          viewy++;
        }
        break;
    case SB_LINEUP :    // Scrolls one line up. 
        if (viewy > conbuf.getmin()) {
          viewy--;
        }
        break;
    case SB_PAGEDOWN :  // Scrolls one page down. 
        if (viewy + 24 < conbuf.getmax()) {     
          viewy += 24;
        } else {
          viewy = conbuf.getmax();
        }
        break;
    case SB_PAGEUP :    // Scrolls one page up. 
        if (viewy > conbuf.getmin() + 24) {
          viewy -= 24;
        } else {
          viewy = conbuf.getmin();
        }
        break;
    case SB_THUMBPOSITION : // The user has dragged the scroll box (thumb) and released the mouse button. The nPos parameter indicates the position of the scroll box at the end of the drag operation. 
        viewy = nPos;
        break;
    case SB_THUMBTRACK :    // The user is dragging the scroll box. This message is sent repeatedly until the user releases the mouse button. The nPos parameter indicates the position that the scroll box has been dragged to. 
        viewy = nPos;
        break;
    case SB_TOP :           // Scrolls to the upper left.
        viewy = 0;
        break;
  }
  if (viewy + 24 == conbuf.getmax()) {
    hotview = true;
  } 
  RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_NOERASE);
}

void ConsoleView::open_log_file()
{
	if(Cloak::logtofile_enabled){
		resized = false;
		cursx = 0;
		cursy = 0;
		viewy = 0;
		hotview = true;

		_error = false;
		lastlf = false;
  
		CString logfile = Cloak::logfile_directory;
		logfile += "cluox_log.log";
		log = fopen(logfile, "a");
		if (log == NULL) {
		  _error = true;
		  err1 = "Cannot open \"cluox_log.log\" for logging.";
		  err2 = "";
		  err3 = "";  
		}
		fputc('\n', log);
		do_log('>', "******************************************************************************");
		std::string msg = " cluox session started at ";
		msg += _strdate(times);
		msg += " ";
		msg += _strtime(times);
		do_log('>', msg.c_str());
		do_log('>', "******************************************************************************");
		do_log(' ', "");
	}
}
