//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  ConsoleView.h
//
//  The sub-view of the black console, handles the cursor position 
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#ifndef __CONSOLE_VIEW_H__
#define __CONSOLE_VIEW_H__

#include <stdio.h>
#include <afxmt.h>
#include "default.h"
#include "ConBuffer.h"

class Cloak;
class MainFrame;

#define WHITE   0xffffff
#define BLUE    0xffA000
#define RED     0x0000ff

class ConsoleView : public CWnd
{
  private :
    int cursx, cursy;
    int viewy;
    SCROLLINFO scrollInfo;
    bool hotview; // autoscroll down in view;
    bool _error;
    bool lastlf;
    CString err1, err2, err3;
    FILE *log;
    char times[100]; // some buffer for diskio

    void do_in(char ch, unsigned int c_color);
    void do_log(char kind, const char *text);
    void newline();

  public :
    ConsoleView();
    ConBuffer conbuf;

    CFont *console_font;
    int line_spacing;
    int x_spacing;
    bool resized;
    Cloak *cloak;
    CMutex mutex_char_in;

    void char_in(char ch);
    void char_in(char ch, unsigned int color);
    void string_in(const char *str);
    void string_in(const char *str, unsigned int color);
    void string_in(const char *str, unsigned int color, unsigned int len);
    void error(const char *text1, const char *text2, const char *text3);
	void open_log_file();

    // Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ConsoleView)
	public:
	protected:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

  public:
    void resize();
    virtual ~ConsoleView();

  protected:
	//{{AFX_MSG(ConsoleView)
	afx_msg void OnPaint();
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}

#endif
