//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  MainFrm.cpp
//
//  The main form.
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#include "stdafx.h"
#include "cluox.h"

#include "MainFrm.h"
#include "Cloak.h"
#include "WatchdogDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Application theApp;

/////////////////////////////////////////////////////////////////////////////
// MainFrame

IMPLEMENT_DYNAMIC(MainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(MainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(MainFrame)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_CHAR()
	ON_WM_CLOSE()
	ON_WM_SIZE()
	ON_MESSAGE(WM_MY_TRAY_NOTIFICATION, OnTrayNotification)
	ON_WM_SHOWWINDOW()
	ON_WM_PAINT()
	ON_WM_GETMINMAXINFO()
	ON_COMMAND(ID_OPTIONS_WATCHDOG, OnOptionsWatchdog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// MainFrame construction/destruction
MainFrame::MainFrame()
{
  Cloak::initalize(&consoleView, this);
}

MainFrame::~MainFrame()
{
}

extern Application theApp;

int MainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  if (CFrameWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }
  // create a view to occupy the client area of the frame
  if (!consoleView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW | WS_VSCROLL,
       CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL)) {
    TRACE0("Failed to create view window\n");
	return -1;
  }

  if (!console_font.CreateFont( 0, 0, 0, 0, FW_NORMAL, false, false, false, ANSI_CHARSET, OUT_RASTER_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FF_DONTCARE, "FixedSys")) {
  //if (!console_font.CreateFont( 0, 0, 0, 0, FW_NORMAL, false, false, false, ANSI_CHARSET, OUT_RASTER_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH, "Terminal")) {
    TRACE0("Failed to create console font\n");
	return -1;
  }  
  consoleView.console_font = &console_font;
  
  if (!m_wndStatusBar.Create(this) || 
      !m_wndStatusBar.SetIndicators(indicators, sizeof(indicators) / sizeof(UINT))) {
    TRACE0("Failed to create status bar\n");
	return -1;
  }

  //scroller.SetScrollRange(0, 100);

  HICON hicon = LoadIcon(theApp.m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME));
  SetIcon(hicon, true);

  minimized = false;

  NOTIFYICONDATA pnid;
  pnid.cbSize = sizeof(NOTIFYICONDATA);
  pnid.hWnd   = m_hWnd;
  pnid.uID    = 0;
  pnid.uFlags = NIF_TIP | NIF_ICON | NIF_MESSAGE;
  pnid.uCallbackMessage = WM_MY_TRAY_NOTIFICATION;
  pnid.hIcon  = hicon;
  strcpy(pnid.szTip, "UOX Server");
  Shell_NotifyIcon(NIM_ADD, &pnid);
   
  Cloak::check_delay     = theApp.GetProfileInt("watchdog", "check_delay",     DEFAULT_CHECK_DELAY);
  Cloak::retries         = theApp.GetProfileInt("watchdog", "retries",         DEFAULT_RETRIES);
  Cloak::retry_delay     = theApp.GetProfileInt("watchdog", "delay_retry",     DEFAULT_RETRY_DELAY);
  Cloak::first_check     = theApp.GetProfileInt("watchdog", "first_delay",     DEFAULT_FIRST_CHECK);
  Cloak::response_time   = theApp.GetProfileInt("watchdog", "response_time",   DEFAULT_RESPONSE_TIME);
  Cloak::watchdog_enable = theApp.GetProfileInt("watchdog", "watchdog_enable", 1) ? true : false;

  Cloak::logfile_directory	= theApp.GetProfileString("options", "log_directory", "");
  Cloak::uox_directory		= theApp.GetProfileString("options", "uox_directory", "");
  Cloak::logtofile_enabled	= theApp.GetProfileInt("options", "enable_logging", 1) ? true : false;
  Cloak::autostart_server	= theApp.GetProfileInt("options", "autostart_server", 0) ? true : false;
  return 0;
}

BOOL MainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
  if( !CFrameWnd::PreCreateWindow(cs) )
	return FALSE;
  cs.style = WS_OVERLAPPED | WS_CAPTION | FWS_ADDTOTITLE | WS_SYSMENU | WS_MINIMIZEBOX;
//  cs.lpszName = "cluox 1.0.1 - UOX-Server Cloak";
  cs.lpszName = "cluox " VERSION " - UOX-Server Cloak";
  cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
  cs.lpszClass = AfxRegisterWndClass(0);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// MainFrame diagnostics

#ifdef _DEBUG
void MainFrame::AssertValid() const
{
  CFrameWnd::AssertValid();
}

void MainFrame::Dump(CDumpContext& dc) const
{
  CFrameWnd::Dump(dc);
}

#endif //_DEBUG

void MainFrame::OnSetFocus(CWnd* pOldWnd)
{
  // ummm nothing :)
}

BOOL MainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
  // This is a very dirty hack, since I simply don't know how to do better.
  // Resize the windows, until the consoleView first repainted, putting it
  // in here get's it that fast so you don't see the flash on beginning;
  if (!consoleView.resized) {
    consoleView.resize();
  }
  // let the view have first crack at the command
  if (consoleView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo)) {
    return TRUE;
  }

  // otherwise, do default handling
  return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void MainFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void MainFrame::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{	
  CFrameWnd::OnKeyUp(nChar, nRepCnt, nFlags);
}

void MainFrame::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  // Keyboard hit, forward them to cloak.
  if (nChar == 13) {
    // translate CR to dos style linefeeds
    Cloak::feed_char(13);
    Cloak::feed_char(10);
  } else {
    Cloak::feed_char(nChar);
  }
  CFrameWnd::OnChar(nChar, nRepCnt, nFlags);
}

void MainFrame::OnClose() 
{
  if (Cloak::cloak_on) {
   // If cloak active shutdown cloak, it will reraise an close
   // message if it is finished.
   Cloak::shutdown();
  }  else {
    // remove the tray icon
    NOTIFYICONDATA pnid;
    pnid.cbSize = sizeof(NOTIFYICONDATA);
    pnid.hWnd   = m_hWnd;
    pnid.uID    = 0;
    pnid.uFlags = 0;
    pnid.uCallbackMessage = NULL;
    pnid.hIcon  = 0;
    pnid.szTip[0] = 0;
    Shell_NotifyIcon(NIM_DELETE, &pnid);
    // and, good-bye cruel world
    CFrameWnd::OnClose();
  }
}


void MainFrame::OnSize(UINT nType, int cx, int cy) 
{
  // Make the main-window invisible, if minimized
  //   (it looks like it "went" into the tray)
  CFrameWnd::OnSize(nType, cx, cy);
  if (nType == SIZE_MINIMIZED) {    
    CloseWindow();
    ::ShowWindow(m_hWnd, SW_HIDE);
    minimized = true;
  }
}


LRESULT MainFrame::OnTrayNotification(WPARAM wID, LPARAM lEvent)
{
  // Make the main-window visible again, if clicked tray icon
  if (lEvent == WM_LBUTTONDBLCLK) {
    ShowWindow(SW_NORMAL);	
    SetForegroundWindow();
    minimized = false;
    return 1; // handled
  }
  return 0; 
}

void MainFrame::OnOptionsWatchdog() 
{
  WatchdogDlg watchDlg;
  if (watchDlg.DoModal() == IDOK) {

    theApp.WriteProfileInt("watchdog", "check_delay", watchDlg.val_delay_watchdog);
    theApp.WriteProfileInt("watchdog", "retries", watchDlg.val_retries);
    theApp.WriteProfileInt("watchdog", "delay_retry", watchDlg.val_delay_retry);
    theApp.WriteProfileInt("watchdog", "first_delay", watchDlg.val_first_delay);
    theApp.WriteProfileInt("watchdog", "response_time", watchDlg.val_respond_time);
    theApp.WriteProfileInt("watchdog", "watchdog_enable", watchDlg.val_watchdog_enable);

    Cloak::check_delay     = watchDlg.val_delay_watchdog;
    Cloak::retries         = watchDlg.val_retries;
    Cloak::retry_delay     = watchDlg.val_delay_retry;
    Cloak::first_check     = watchDlg.val_first_delay;
    Cloak::response_time   = watchDlg.val_respond_time;
    Cloak::watchdog_enable = watchDlg.val_watchdog_enable ? true : false;
  }
}