//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox.cpp
//
//  The main file
//
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//  cluox - Windows-GUI wrapper for UOX.
//  Copyright (C) 2000, 2001 Axel Kittenberger
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

#include "stdafx.h"
#include "cluox.h"

#include "AboutDlg.h"
#include "MainFrm.h"
#include "Options.h"
#include "Cloak.h"

BEGIN_MESSAGE_MAP(Application, CWinApp)
	//{{AFX_MSG_MAP(Application)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_OPTIONSDLG, OnDlgOptions)
	ON_COMMAND(ID_STARTSERVER, OnStartServer)
	ON_COMMAND(ID_SHUTDOWNSERVER, OnShutdownServer)
	ON_COMMAND(ID_RELOAD_INI, OnReloadINI)
	ON_COMMAND(ID_RELOAD_ACCOUNTS, OnReloadAccounts)
	ON_COMMAND(ID_RELOAD_REGIONS, OnReloadRegions)
	ON_COMMAND(ID_RELOAD_SPAWNREGIONS, OnReloadSpawnRegions)
	ON_COMMAND(ID_RELOAD_SPELLS, OnReloadSpells)
	ON_COMMAND(ID_RELOAD_DFNFILES, OnReloadDFNFiles)
	ON_COMMAND(ID_RELOAD_JSFILES, OnReloadJSFiles)
	ON_COMMAND(ID_SECURECONSOLE, OnSecureConsole)
	ON_COMMAND(ID_COMMANDLIST, OnCommandList)
	ON_COMMAND(ID_CONFIGURATION, OnConfiguration)
	ON_COMMAND(ID_CONSOLEBROADCAST, OnConsoleBroadcast)
	ON_COMMAND(ID_SAVE_ACCOUNTS, OnSaveAccounts)
	ON_COMMAND(ID_SAVE_WORLD, OnSaveWorld)
	ON_COMMAND(ID_MAIN_PERFORMANCE, OnPerformance)
	ON_COMMAND(ID_MAIN_CHARACTERS, OnCharcters)
	ON_COMMAND(ID_MAIN_MEMORY, OnMemory)
	ON_COMMAND(ID_MAIN_SHUTDOWN, OnShutdown)
	ON_COMMAND(ID_MAIN_DUMPLOOKUPS, OnDumpLookups)
	ON_COMMAND(ID_MAIN_DISPLAYMAPS, OnDisplayMaps)
	ON_COMMAND(ID_NETWORK_DISCONZERO, OnDisconnectZero)
	ON_COMMAND(ID_NETWORK_DISCONALL, OnDisconnectAll)
	ON_COMMAND(ID_SOCKETLOGGING, OnSocketLogging)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

Application::Application()
{
}

Application theApp;

BOOL Application::InitInstance()
{
	// Standard initialization
#ifdef _AFXDLL
  Enable3dControls();       // Call this when using MFC in a shared DLL
#else
  Enable3dControlsStatic(); // Call this when linking to MFC statically
#endif
  // Change the registry key under which the settings are stored.
  SetRegistryKey("cluox");

  // tell windows we want to shutdown first, before it pesters UOX.
  // however windows does'nt seem to respect this :(
  int _asw = SetProcessShutdownParameters(0x380, 0);
  
  MainFrame* pFrame = new MainFrame;
  m_pMainWnd = pFrame;
  // create and load the frame with its resources
  pFrame->LoadFrame(IDR_MAINFRAME,
		            WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, 
                    NULL,
		            NULL
                   );

  pFrame->ShowWindow(SW_SHOW);
  pFrame->UpdateWindow();
  return TRUE;
}

void Application::OnAppAbout()
{
  AboutDlg aboutDlg;
  aboutDlg.DoModal();
}

void Application::OnDlgOptions() // Options Menu - ShadowBranch
{
  OptionsDlg optionDlg;
  if(optionDlg.DoModal() == IDOK) {
	theApp.WriteProfileString("options", "log_directory", optionDlg.log_directory);
	theApp.WriteProfileString("options", "uox_directory", optionDlg.uox_directory);
	theApp.WriteProfileInt("options", "enable_logging", optionDlg.logging_active);
	theApp.WriteProfileInt("options", "autostart_server", optionDlg.autostart_server);

    Cloak::logfile_directory	= optionDlg.log_directory;
    Cloak::uox_directory		= optionDlg.uox_directory;
    Cloak::logtofile_enabled	= optionDlg.logging_active ? true : false;
	Cloak::autostart_server		= optionDlg.autostart_server ? true : false;
  }
}

void Application::OnStartServer()
{
	Cloak::startserver();
}

void Application::OnShutdownServer()
{
	if(Cloak::cloak_on){
		Cloak::shutdown();
	}
}

void Application::OnReloadAccounts()
{
	Cloak::reloadaccounts();
}

void Application::OnReloadDFNFiles()
{
	Cloak::reloaddfnfiles();
}

void Application::OnReloadINI()
{
	Cloak::reloadini();
}

void Application::OnReloadJSFiles()
{
	Cloak::reloadjsfiles();
}

void Application::OnReloadRegions()
{
	Cloak::reloadregions();
}

void Application::OnReloadSpawnRegions()
{
	Cloak::reloadspawnregions();
}

void Application::OnReloadSpells()
{
	Cloak::reloadspells();
}

void Application::OnSecureConsole()
{
	Cloak::secureconsole();
}

void Application::OnCommandList()
{
	Cloak::commandlist();
}

void Application::OnConfiguration()
{
	Cloak::configuration();
}

void Application::OnConsoleBroadcast()
{
	Cloak::consolebroadcast();
}

void Application::OnSaveAccounts()
{
	Cloak::save_accounts();
}

void Application::OnSaveWorld()
{
	Cloak::save_world();
}

void Application::OnPerformance()
{
	Cloak::maintain_performance();
}

void Application::OnCharcters()
{
	Cloak::maintain_performance();
}

void Application::OnMemory()
{
	Cloak::maintain_memory();
}

void Application::OnShutdown()
{
	Cloak::maintain_shutdown();
}

void Application::OnDumpLookups()
{
	Cloak::maintain_lookups();
}

void Application::OnDisplayMaps()
{
	Cloak::maintain_maps();
}

void Application::OnDisconnectZero()
{
	Cloak::network_disconzero();
}

void Application::OnDisconnectAll()
{
	Cloak::network_disconall();
}

void Application::OnSocketLogging()
{
	Cloak::network_socketlogging();
}
